package com.tzld.piaoquan.ad.engine.service.log.impl;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.tzld.commons.aliyun.log.AliyunLogManager;
import com.tzld.piaoquan.ad.engine.commons.score.ScoreParam;
import com.tzld.piaoquan.ad.engine.service.log.LogHubService;
import com.tzld.piaoquan.ad.engine.service.score.dto.AdPlatformCreativeDTO;
import com.tzld.piaoquan.ad.engine.service.score.param.RecommendRequestParam;
import com.tzld.piaoquan.recommend.feature.domain.ad.base.AdRankItem;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections4.CollectionUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import java.util.*;
import java.util.concurrent.LinkedBlockingDeque;
import java.util.concurrent.ThreadPoolExecutor;
import java.util.concurrent.TimeUnit;
import java.util.function.Function;
import java.util.stream.Collectors;

@Slf4j
@Service
public class LogHubServiceImpl implements LogHubService {

    private static final ThreadPoolExecutor logUploadThreadPool = new ThreadPoolExecutor(20, 40, 30,
            TimeUnit.MINUTES, new LinkedBlockingDeque<>(), new ThreadPoolExecutor.CallerRunsPolicy());

    @Value("${aliyun.log.logstore.statistics}")
    private String logStore;
    @Value("${aliyun.log.project}")
    private String project;

    @Autowired
    private AliyunLogManager aliyunLogManager;

    @Override
    public void scoreLogUpload(ScoreParam param, List<AdPlatformCreativeDTO> adIdList, List<AdRankItem> rankItems, RecommendRequestParam requestParam, String scoreStrategy, String abCode) {
        logUploadThreadPool.execute(new Runnable() {
            @Override
            public void run() {
                JSONObject logMap = new JSONObject();
                logMap.put("pqtId", param.getPqtId());
                logMap.put("mid", param.getMid());
                logMap.put("videoId", param.getVideoId());
                logMap.put("abCode", abCode);
                // 获取AB实验列表
                Set<String> abExpCode = new HashSet<>();
                if (CollectionUtils.isNotEmpty(requestParam.getAdAbExpArr())) {
                    for (Map map : requestParam.getAdAbExpArr()) {
                        if (map.containsKey("abExpCode")) {
                            abExpCode.add(map.get("abExpCode").toString());
                        }
                    }
                }
                logMap.put("abExpCode", abExpCode);

                List<JSONObject> scoreResult = new ArrayList<>();
                for (AdRankItem rankItem : rankItems) {
                    JSONObject json = new JSONObject();
                    json.put("adId", rankItem.getAdId());
                    json.put("score", rankItem.getScore());
                    json.put("ext", rankItem.getExt());
                    scoreResult.add(json);
                }
                logMap.put("scoreResult", JSON.toJSONString(scoreResult));

                AdRankItem top1 = rankItems.get(0);
                logMap.put("top1_adId", top1.getAdId());
                logMap.put("top1_score", top1.getScore());
                logMap.put("top1_exp", top1.getExt());

                logMap.put("creativeList", JSON.toJSONString(adIdList));
                logMap.put("adAbGroup", requestParam.getAdAbGroup());
                logMap.put("scoreStrategy", scoreStrategy);
                logMap.put("appType", requestParam.getAppType());
                aliyunLogManager.sendLog(project, logStore, "", logMap);
            }
        });
    }
}
